#include "xil_types.h"

#define code
#include "register_map.h"

#include "si5338_i2c.h"

#define SI5338_DELAY 28800 //12ms

#define LOCK_MASK 0x15
#define LOS_MASK 0x04



int si5338_init() {
	int counter;
	unsigned char curr_chip_val, clear_curr_val, clear_new_val, combined, reg;

	Reg_Data curr;

	I2C_ByteWrite(230, 0x10);                   //OEB_ALL = 1
	I2C_ByteWrite(241, 0xE5);                   //DIS_LOL = 1
	for (counter=0;counter<NUM_REGS_MAX;counter++) {
		curr = Reg_Store[counter];
		if (curr.Reg_Mask==0xFF) {
			I2C_ByteWrite(curr.Reg_Addr, curr.Reg_Val);

		} else {
			curr_chip_val = I2C_ByteRead(curr.Reg_Addr);
			clear_curr_val = curr_chip_val & ~curr.Reg_Mask;
			clear_new_val = curr.Reg_Val & curr.Reg_Mask;
			combined = clear_new_val | clear_curr_val;
			I2C_ByteWrite(curr.Reg_Addr, combined);
		}

	}


	// check LOS alarm for the xtal input
	// on IN1 and IN2 (and IN3 if necessary) -
	// change this mask if using inputs on IN4, IN5, IN6
	reg = I2C_ByteRead(218) & LOS_MASK;
	while(reg != 0){
		reg = I2C_ByteRead(218) & LOS_MASK;
	}

	I2C_ByteWrite(49, I2C_ByteRead(49) & 0x7F); //FCAL_OVRD_EN = 0
	I2C_ByteWrite(246, 2);                      //soft reset
	I2C_ByteWrite(241, 0x65);                   //DIS_LOL = 0


	// wait for Si5338 to be ready after calibration (ie, soft reset)
	//counter = 0;	while(counter < SI5338_DELAY) { counter++; }
	//counter = 0;	while(counter < SI5338_DELAY) { counter++; }

	delay(100);

	//make sure the device locked by checking PLL_LOL and SYS_CAL
	reg = I2C_ByteRead(218) & LOCK_MASK;
	while(reg != 0){
		reg = I2C_ByteRead(218) & LOCK_MASK;

	}

	//copy FCAL values
	I2C_ByteWrite(45, I2C_ByteRead(235));
	I2C_ByteWrite(46, I2C_ByteRead(236));
	// clear bits 0 and 1 from 47 and
	// combine with bits 0 and 1 from 237
	reg = (I2C_ByteRead(47) & 0xFC) | (I2C_ByteRead(237) & 3);
	I2C_ByteWrite(47, reg);

	I2C_ByteWrite(49, I2C_ByteRead(49) | 0x80); // FCAL_OVRD_EN = 1
	I2C_ByteWrite(230, 0x00);                   // OEB_ALL = 0

	return 0;
}

